@@ -1,7 +1,7 @@
 import hashlib
 import pickle
 import random
-from typing import Tuple, Any, Callable, List, Optional
+from typing import Tuple, Any, Callable, List, Optional, Union, Sequence
 
 from fakeredis import _msgs as msgs
 from fakeredis._command_args_parsing import extract_args
@@ -17,16 +17,17 @@ from fakeredis._commands import (
 )
 from fakeredis._helpers import compile_pattern, SimpleError, OK, casematch, Database, SimpleString
 from fakeredis.model import ZSet, Hash, ExpiringMembersSet
+from fakeredis.typing import VersionType
 
 
 class GenericCommandsMixin:
     _ttl: Callable[[CommandItem, float], int]
-    _scan: Callable[[CommandItem, int, bytes, bytes], Tuple[int, List[bytes]]]
+    _scan: Callable[[Sequence[bytes], int, bytes], List[Union[bytes, List[bytes]]]]
     _key_value_type: Callable[[CommandItem], SimpleString]
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         super(GenericCommandsMixin, self).__init__(*args, **kwargs)
-        self.version: Tuple[int]
+        self.version: VersionType
         self._server: Any
         self._db: Database
         self._db_num: int
@@ -61,11 +62,14 @@ class GenericCommandsMixin:
 
     def _expireat(self, key: CommandItem, timestamp: float, *args: bytes) -> int:
         (
-            nx,
-            xx,
-            gt,
-            lt,
-        ), _ = extract_args(
+            (
+                nx,
+                xx,
+                gt,
+                lt,
+            ),
+            _,
+        ) = extract_args(
             args,
             (
                 "nx",
@@ -162,15 +166,15 @@ class GenericCommandsMixin:
             return -2
         if key.expireat is None:
             return -1
-        return key.expireat
+        return int(key.expireat)
 
     @command(name="PEXPIRETIME", fixed=(Key(),))
-    def pexpiretime(self, key: CommandItem) -> float:
+    def pexpiretime(self, key: CommandItem) -> int:
         if key.value is None:
             return -2
         if key.expireat is None:
             return -1
-        return key.expireat * 1000
+        return int(key.expireat * 1000)
 
     @command(name="RANDOMKEY", fixed=())
     def randomkey(self):
@@ -226,13 +230,16 @@ class GenericCommandsMixin:
         if key.value is not None and not isinstance(key.value, (ExpiringMembersSet, list, ZSet)):
             raise SimpleError(msgs.WRONGTYPE_MSG)
         (
-            asc,
-            desc,
-            alpha,
-            store,
-            sortby,
-            (limit_start, limit_count),
-        ), left_args = extract_args(
+            (
+                asc,
+                desc,
+                alpha,
+                store,
+                sortby,
+                (limit_start, limit_count),
+            ),
+            left_args,
+        ) = extract_args(
             args,
             ("asc", "desc", "alpha", "*store", "*by", "++limit"),
             error_on_unexpected=False,
@@ -307,12 +314,15 @@ class GenericCommandsMixin:
         if key.value is not None and not isinstance(key.value, (set, list, ZSet)):
             raise SimpleError(msgs.WRONGTYPE_MSG)
         (
-            asc,
-            desc,
-            alpha,
-            sortby,
-            (limit_start, limit_count),
-        ), left_args = extract_args(
+            (
+                asc,
+                desc,
+                alpha,
+                sortby,
+                (limit_start, limit_count),
+            ),
+            left_args,
+        ) = extract_args(
             args,
             ("asc", "desc", "alpha", "*by", "++limit"),
             error_on_unexpected=False,
@@ -385,3 +395,18 @@ class GenericCommandsMixin:
     @command(name="UNLINK", fixed=(Key(),), repeat=(Key(),))
     def unlink(self, *keys: CommandItem) -> int:
         return delete_keys(*keys)
+
+    @command(name="COPY", fixed=(Key(), Key()), repeat=(bytes,))
+    def copy(self, key: CommandItem, newkey: CommandItem, *args: bytes) -> int:
+        (db_num, replace), _ = extract_args(args, ("+db", "replace"))
+        db_num = db_num or self._db_num
+        if key.key == newkey.key and db_num == self._db_num:
+            raise SimpleError(msgs.SRC_DST_SAME_MSG)
+        if (newkey.key in self._server.dbs[db_num] and not replace) or (key.key not in self._server.dbs[self._db_num]):
+            return 0
+
+        newkey.value = key.value
+        newkey.expireat = key.expireat
+        self._server.dbs[db_num][newkey.key] = key
+        newkey.db = self._server.dbs[db_num]
+        return 1
